@@ -2,8 +2,6 @@ from __future__ import annotations
 
 import numpy as np
 
-from ada.base import Backend
-
 
 class Node:
     """
@@ -67,7 +65,9 @@ class Node:
     @units.setter
     def units(self, value):
         if value != self._units:
-            scale_factor = Backend._unit_conversion(self._units, value)
+            from ada.core.utils import unit_length_conversion
+
+            scale_factor = unit_length_conversion(self._units, value)
             self.p *= scale_factor
             if self._r is not None:
                 self._r *= scale_factor
